/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.modules.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.modules.entity.ModuleChoose;
import com.jeeplus.modules.modules.service.ModuleChooseService;

/**
 * 模块选择Controller
 * @author 李韵轩
 * @version 2017-05-02
 */
@Controller
@RequestMapping(value = "${adminPath}/modules/moduleChoose")
public class ModuleChooseController extends BaseController {

	@Autowired
	private ModuleChooseService moduleChooseService;
	
	@ModelAttribute
	public ModuleChoose get(@RequestParam(required=false) String id) {
		ModuleChoose entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = moduleChooseService.get(id);
		}
		if (entity == null){
			entity = new ModuleChoose();
		}
		return entity;
	}
	
	/**
	 * 模块列表页面
	 */
	@RequiresPermissions("modules:moduleChoose:list")
	@RequestMapping(value = {"list", ""})
	public String list(ModuleChoose moduleChoose, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<ModuleChoose> page = moduleChooseService.findPage(new Page<ModuleChoose>(request, response), moduleChoose); 
		model.addAttribute("page", page);
		return "modules/modules/moduleChooseList";
	}

	/**
	 * 查看，增加，编辑模块表单页面
	 */
	@RequiresPermissions(value={"modules:moduleChoose:view","modules:moduleChoose:add","modules:moduleChoose:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(ModuleChoose moduleChoose, Model model) {
		model.addAttribute("moduleChoose", moduleChoose);
		return "modules/modules/moduleChooseForm";
	}

	/**
	 * 保存模块
	 */
	@RequiresPermissions(value={"modules:moduleChoose:add","modules:moduleChoose:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(ModuleChoose moduleChoose, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, moduleChoose)){
			return form(moduleChoose, model);
		}
		if(!moduleChoose.getIsNewRecord()){//编辑表单保存
			ModuleChoose t = moduleChooseService.get(moduleChoose.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(moduleChoose, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			moduleChooseService.save(t);//保存
		}else{//新增表单保存
			moduleChooseService.save(moduleChoose);//保存
		}
		addMessage(redirectAttributes, "保存模块成功");
		return "redirect:"+Global.getAdminPath()+"/modules/moduleChoose/?repage";
	}
	
	/**
	 * 删除模块
	 */
	@RequiresPermissions("modules:moduleChoose:del")
	@RequestMapping(value = "delete")
	public String delete(ModuleChoose moduleChoose, RedirectAttributes redirectAttributes) {
		moduleChooseService.delete(moduleChoose);
		addMessage(redirectAttributes, "删除模块成功");
		return "redirect:"+Global.getAdminPath()+"/modules/moduleChoose/?repage";
	}
	
	/**
	 * 批量删除模块
	 */
	@RequiresPermissions("modules:moduleChoose:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			moduleChooseService.delete(moduleChooseService.get(id));
		}
		addMessage(redirectAttributes, "删除模块成功");
		return "redirect:"+Global.getAdminPath()+"/modules/moduleChoose/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("modules:moduleChoose:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(ModuleChoose moduleChoose, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "模块"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<ModuleChoose> page = moduleChooseService.findPage(new Page<ModuleChoose>(request, response, -1), moduleChoose);
    		new ExportExcel("模块", ModuleChoose.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出模块记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/modules/moduleChoose/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("modules:moduleChoose:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<ModuleChoose> list = ei.getDataList(ModuleChoose.class);
			for (ModuleChoose moduleChoose : list){
				try{
					moduleChooseService.save(moduleChoose);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条模块记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条模块记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模块失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/modules/moduleChoose/?repage";
    }
	
	/**
	 * 下载导入模块数据模板
	 */
	@RequiresPermissions("modules:moduleChoose:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "模块数据导入模板.xlsx";
    		List<ModuleChoose> list = Lists.newArrayList(); 
    		new ExportExcel("模块数据", ModuleChoose.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/modules/moduleChoose/?repage";
    }
	
	
	

}