/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.serialnum.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import com.jeeplus.modules.sys.utils.UserUtils;
import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.serialnum.entity.SerialNumTpl;
import com.jeeplus.modules.serialnum.service.SerialNumTplService;

/**
 * 编号模板Controller
 * @author ppt
 * @version 2018-05-16
 */
@Controller
@RequestMapping(value = "${adminPath}/serialnum/serialNumTpl")
public class SerialNumTplController extends BaseController {

	@Autowired
	private SerialNumTplService serialNumTplService;
	
	@ModelAttribute
	public SerialNumTpl get(@RequestParam(required=false) String id) {
		SerialNumTpl entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = serialNumTplService.get(id);
		}
		if (entity == null){
			entity = new SerialNumTpl();
		}
		return entity;
	}
	
	/**
	 * 编号模板列表页面
	 */
	@RequiresPermissions("serialnum:serialNumTpl:list")
	@RequestMapping(value = {"list", ""})
	public String list(SerialNumTpl serialNumTpl, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<SerialNumTpl> page = serialNumTplService.findPage(new Page<SerialNumTpl>(request, response), serialNumTpl);
		model.addAttribute("page", page);
		return "modules/serialnum/serialNumTplList";
	}

	/**
	 * 增加，编辑编号模板表单页面
	 */
	@RequiresPermissions(value={"serialnum:serialNumTpl:add","serialnum:serialNumTpl:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(SerialNumTpl serialNumTpl, Model model) {
		if (serialNumTpl!=null&&StringUtils.isNotBlank(serialNumTpl.getId()))serialNumTpl = serialNumTplService.get(serialNumTpl.getId());
		model.addAttribute("serialNumTpl", serialNumTpl);
		return "modules/serialnum/serialNumTplForm";
	}

	/**
	 * 查看编号模板表单页面
	 */
	@RequiresPermissions("serialnum:serialNumTpl:list")
	@RequestMapping(value = "view")
	public String view(SerialNumTpl serialNumTpl, Model model) {
		if (serialNumTpl!=null&&StringUtils.isNotBlank(serialNumTpl.getId()))serialNumTpl = serialNumTplService.get(serialNumTpl.getId());
		model.addAttribute("serialNumTpl", serialNumTpl);
		return "modules/serialnum/serialNumTplView";
	}

	/**
	 * 保存编号模板
	 */
	@RequiresPermissions(value={"serialnum:serialNumTpl:add","serialnum:serialNumTpl:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(SerialNumTpl serialNumTpl, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, serialNumTpl)){
			return form(serialNumTpl, model);
		}
		try {
            if (!serialNumTpl.getIsNewRecord()) {//编辑表单保存
                SerialNumTpl t = serialNumTplService.get(serialNumTpl.getId());//从数据库取出记录的值
                if (t.getSerialNum() > serialNumTpl.getSerialNum()) {
                    addMessage(redirectAttributes, "保存编号模板失败：序列号发生变化，起始序列号不能小于原有序列号");
                    return "redirect:" + Global.getAdminPath() + "/serialnum/serialNumTpl/?repage";
                }
                t.setOldSerialNum(t.getSerialNum());
                MyBeanUtils.copyBeanNotNull2Bean(serialNumTpl, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
                int edit = serialNumTplService.edit(t);//保存
                if (edit == 0) {
                    addMessage(redirectAttributes, "保存编号模板失败：序列号发生变化，更新模板失败。请重试");
                    return "redirect:" + Global.getAdminPath() + "/serialnum/serialNumTpl/?repage";
                }
            } else {//新增表单保存
                serialNumTplService.save(serialNumTpl);//保存
            }
            addMessage(redirectAttributes, "保存编号模板成功");
        }catch (DuplicateKeyException e){
		    logger.error("保存编号模板失败 ",e);
            addMessage(redirectAttributes, "保存编号模板失败：该公司已存在相同业务类型的编号模板");
        }catch (Exception e){
            logger.error("保存编号模板失败 ",e);
            addMessage(redirectAttributes, "保存编号模板失败："+e.getMessage());
        }
		return "redirect:"+Global.getAdminPath()+"/serialnum/serialNumTpl/?repage";
	}
	
	/**
	 * 删除编号模板
	 */
	@RequiresPermissions("serialnum:serialNumTpl:del")
	@RequestMapping(value = "delete")
	public String delete(SerialNumTpl serialNumTpl, RedirectAttributes redirectAttributes) {
		serialNumTplService.delete(serialNumTpl);
		addMessage(redirectAttributes, "删除编号模板成功");
		return "redirect:"+Global.getAdminPath()+"/serialnum/serialNumTpl/?repage";
	}
	
	/**
	 * 批量删除编号模板
	 */
	@RequiresPermissions("serialnum:serialNumTpl:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			serialNumTplService.delete(serialNumTplService.get(id));
		}
		addMessage(redirectAttributes, "删除编号模板成功");
		return "redirect:"+Global.getAdminPath()+"/serialnum/serialNumTpl/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("serialnum:serialNumTpl:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(SerialNumTpl serialNumTpl, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "编号模板"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<SerialNumTpl> page = serialNumTplService.findPage(new Page<SerialNumTpl>(request, response, -1), serialNumTpl);
    		new ExportExcel("编号模板", SerialNumTpl.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出编号模板记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/serialnum/serialNumTpl/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("serialnum:serialNumTpl:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<SerialNumTpl> list = ei.getDataList(SerialNumTpl.class);
			for (SerialNumTpl serialNumTpl : list){
				try{
					serialNumTplService.save(serialNumTpl);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条编号模板记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条编号模板记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入编号模板失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/serialnum/serialNumTpl/?repage";
    }
	
	/**
	 * 下载导入编号模板数据模板
	 */
	@RequiresPermissions("serialnum:serialNumTpl:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "编号模板数据导入模板.xlsx";
    		List<SerialNumTpl> list = Lists.newArrayList(); 
    		new ExportExcel("编号模板数据", SerialNumTpl.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/serialnum/serialNumTpl/?repage";
    }
	
	
	

}