/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.sysparameter.service;

import java.util.ArrayList;
import java.util.List;

import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.modules.sys.entity.Office;
import com.jeeplus.modules.sys.utils.UserUtils;
import com.jeeplus.modules.sysparameter.dao.SysParameterValueDao;
import com.jeeplus.modules.sysparameter.entity.SysParameterValue;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.service.CrudService;
import com.jeeplus.modules.sysparameter.entity.SysParameter;
import com.jeeplus.modules.sysparameter.dao.SysParameterDao;

/**
 * 系统参数Service
 * @author chenyuping
 * @version 2018-08-13
 */
@Service
@Transactional(readOnly = true)
public class SysParameterService extends CrudService<SysParameterDao, SysParameter> {
	@Autowired
	private SysParameterValueDao sysParameterValueDao;
	public SysParameter get(String id) {
		return super.get(id);
	}
	
	public List<SysParameter> findList(SysParameter sysParameter) {
		return super.findList(sysParameter);
	}
	
	public Page<SysParameter> findPage(Page<SysParameter> page, SysParameter sysParameter) {
		sysParameter.setPage(page);
		page.setList(this.findList(sysParameter));
		return page;
	}
	
	@Transactional(readOnly = false)
	public void save(SysParameter sysParameter) {
		if(UserUtils.getUser().isAdmin()){
			super.save(sysParameter);
		}
		this.saveValue(sysParameter);
	}
	
	@Transactional(readOnly = false)
	public void delete(SysParameter sysParameter) {
		super.delete(sysParameter);
	}


	/**
	 * 保存参数值
	 * @param sysParameter
	 */
	public void saveValue(SysParameter sysParameter){
		if(UserUtils.getUser().isAdmin()){//管理员添加参数值为默认值
			SysParameterValue sysParameterValue=sysParameterValueDao.findByParameter(sysParameter.getId());
			if(sysParameterValue==null){
				SysParameterValue parameterValue=new SysParameterValue();
				parameterValue.preInsert();
				parameterValue.setBranchOffice("1");
				parameterValue.setCompanyId("1");
				parameterValue.setFalt("1");
				parameterValue.setParameterId(sysParameter.getId());
				parameterValue.setValue(sysParameter.getParameterValue());
				sysParameterValueDao.insert(parameterValue);
			}else{
				sysParameterValue.preUpdate();
				sysParameterValue.setBranchOffice("1");
				sysParameterValue.setCompanyId("1");
				sysParameterValue.setFalt("1");
				sysParameterValue.setParameterId(sysParameter.getId());
				sysParameterValue.setValue(sysParameter.getParameterValue());
				sysParameterValueDao.update(sysParameterValue);
			}
		}else{
			String branchOffice=sysParameter.getCompany();
			SysParameterValue sysParameterValue=sysParameterValueDao.findByParameterFalt(sysParameter.getId(),branchOffice);
			if(sysParameterValue==null){
				SysParameterValue parameterValue=new SysParameterValue();
				parameterValue.preInsert();
				parameterValue.setBranchOffice(branchOffice);
				parameterValue.setCompanyId(UserUtils.getSelectCompany().getId());
				parameterValue.setFalt("0");
				parameterValue.setParameterId(sysParameter.getId());
				parameterValue.setValue(sysParameter.getParameterValue());
				sysParameterValueDao.insert(parameterValue);
			}else {
				sysParameterValue.preUpdate();
				sysParameterValue.setBranchOffice(branchOffice);
				sysParameterValue.setCompanyId(UserUtils.getSelectCompany().getId());
				sysParameterValue.setFalt("0");
				sysParameterValue.setParameterId(sysParameter.getId());
				sysParameterValue.setValue(sysParameter.getParameterValue());
				sysParameterValueDao.update(sysParameterValue);
			}

		}

	}
}