/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.sysparameter.web;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import com.jeeplus.modules.sys.dao.OfficeDao;
import com.jeeplus.modules.sys.dao.RoleDao;
import com.jeeplus.modules.sys.entity.Office;
import com.jeeplus.modules.sys.entity.Role;
import com.jeeplus.modules.sys.utils.UserUtils;
import com.jeeplus.modules.sysparameter.dao.SysParameterDao;
import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.sysparameter.entity.SysParameter;
import com.jeeplus.modules.sysparameter.service.SysParameterService;

/**
 * 系统参数Controller
 * @author chenyuping
 * @version 2018-08-13
 */
@Controller
@RequestMapping(value = "${adminPath}/sysparameter/sysParameter")
public class SysParameterController extends BaseController {
	@Autowired
	private HttpServletRequest request;
	@Autowired
	private SysParameterService sysParameterService;
	@Autowired
	private SysParameterDao sysParameterDao;
	@Autowired
	private RoleDao roleDao;
	@Autowired
	private OfficeDao officeDao;

	@ModelAttribute
	public SysParameter get(@RequestParam(required=false) String id) {
		SysParameter entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = sysParameterService.get(id);
		}
		if (entity == null){
			entity = new SysParameter();
		}
		return entity;
	}
	
	/**
	 * 系统参数列表页面
	 */
	@RequiresPermissions("sysparameter:sysParameter:list")
	@RequestMapping(value = {"list", ""})
	public String list(SysParameter sysParameter, HttpServletRequest request, HttpServletResponse response, Model model) {
		boolean admin = UserUtils.getUser().isAdmin();
		List<Role> roleList=new ArrayList<>();
		if(admin){
			Role role=new Role();
			Office office=new Office();
			office.setId("1");
			office.setName("系统默认");
			role.setOffice(office);
			roleList.add(role);
		}else{
			roleList =  roleDao.getManagerRoleList(UserUtils.getUser().getId(),UserUtils.getSelectCompany().getId());
		}
		Page<SysParameter> page = sysParameterService.findPage(new Page<SysParameter>(request, response), sysParameter); 
		model.addAttribute("page", page);
		model.addAttribute("list", roleList);
		return "modules/sysparameter/sysParameterList";
	}

	/**
	 * 查看，增加，编辑系统参数表单页面
	 */
	@RequiresPermissions(value={"sysparameter:sysParameter:view","sysparameter:sysParameter:add","sysparameter:sysParameter:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(SysParameter sysParameter, Model model) {
     String  tbl=request.getParameter("tabId");
     String company= request.getParameter("company");
     Office office=officeDao.get(company);
		sysParameter.setCompany(company);
		sysParameter.setCompanyName(office.getName());
      if("1".equals(tbl)){
	 model.addAttribute("sysParameter", sysParameter);
	 return "modules/sysparameter/sysParameterAdminForm";
      }
		if(UserUtils.getUser().isAdmin()){//管理员回显系统默认参数值
			SysParameter s = sysParameterDao.findByParameter(sysParameter.getId());
			sysParameter.setParameterValue(s.getParameterValue());
			model.addAttribute("sysParameter", sysParameter);
			return "modules/sysparameter/sysParameterAdminForm";
		}else{//当前登录人为分公司人员查看总公司与系统默认值
			if("3".equals(office.getType())){
				SysParameter s = sysParameterDao.findByParameterFalt(sysParameter.getId(),company);
				if(s!=null){
					sysParameter.setParameterValue(s.getParameterValue());
				}
				model.addAttribute("officeType", "3");
				SysParameter s1 = sysParameterDao.findByParameterFalt(sysParameter.getId(),UserUtils.getSelectCompany().getId());
				if(s1!=null){
					sysParameter.setComValue(s1.getParameterValue());
				}else{
					sysParameter.setComValue("未定义");
				}
				SysParameter sys = sysParameterDao.findByParameter(sysParameter.getId());
				sysParameter.setSysValue(sys.getParameterValue());
				model.addAttribute("sysParameter", sysParameter);
				return "modules/sysparameter/sysParameterForm";
			}else{//当前登录人为总公司人员查看系统默认值
				model.addAttribute("officeType", "1");
				SysParameter s1 = sysParameterDao.findByParameterFalt(sysParameter.getId(),company);
				if(s1!=null){
					sysParameter.setParameterValue(s1.getParameterValue());
				}
				SysParameter sys = sysParameterDao.findByParameter(sysParameter.getId());
				sysParameter.setSysValue(sys.getParameterValue());
				model.addAttribute("sysParameter", sysParameter);
				return "modules/sysparameter/sysParameterForm";
			}

		}

	}

	/**
	 * 保存系统参数
	 */
	@RequiresPermissions(value={"sysparameter:sysParameter:add","sysparameter:sysParameter:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(SysParameter sysParameter, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, sysParameter)){
			return form(sysParameter, model);
		}
		if(UserUtils.isManager()){
			if(!sysParameter.getIsNewRecord()){//编辑表单保存
				SysParameter t = sysParameterService.get(sysParameter.getId());//从数据库取出记录的值
				MyBeanUtils.copyBeanNotNull2Bean(sysParameter, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
				sysParameterService.save(t);//保存
			}else{//新增表单保存
				sysParameterService.save(sysParameter);//保存
			}
			addMessage(redirectAttributes, "保存系统参数成功");
			return "redirect:"+Global.getAdminPath()+"/sysparameter/sysParameter/?repage";
		}else{
			addMessage(redirectAttributes, "只有管理员才有操作权限！");
			return "redirect:"+Global.getAdminPath()+"/sysparameter/sysParameter/?repage";
		}

	}
	
	/**
	 * 删除系统参数
	 */
	@RequiresPermissions("sysparameter:sysParameter:del")
	@RequestMapping(value = "delete")
	public String delete(SysParameter sysParameter, RedirectAttributes redirectAttributes) {
		sysParameterService.delete(sysParameter);
		addMessage(redirectAttributes, "删除系统参数成功");
		return "redirect:"+Global.getAdminPath()+"/sysparameter/sysParameter/?repage";
	}
	
	/**
	 * 批量删除系统参数
	 */
	@RequiresPermissions("sysparameter:sysParameter:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			sysParameterService.delete(sysParameterService.get(id));
		}
		addMessage(redirectAttributes, "删除系统参数成功");
		return "redirect:"+Global.getAdminPath()+"/sysparameter/sysParameter/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("sysparameter:sysParameter:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(SysParameter sysParameter, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "系统参数"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<SysParameter> page = sysParameterService.findPage(new Page<SysParameter>(request, response, -1), sysParameter);
    		new ExportExcel("系统参数", SysParameter.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出系统参数记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/sysparameter/sysParameter/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("sysparameter:sysParameter:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<SysParameter> list = ei.getDataList(SysParameter.class);
			for (SysParameter sysParameter : list){
				try{
					sysParameterService.save(sysParameter);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条系统参数记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条系统参数记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入系统参数失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/sysparameter/sysParameter/?repage";
    }
	
	/**
	 * 下载导入系统参数数据模板
	 */
	@RequiresPermissions("sysparameter:sysParameter:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "系统参数数据导入模板.xlsx";
    		List<SysParameter> list = Lists.newArrayList(); 
    		new ExportExcel("系统参数数据", SysParameter.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/sysparameter/sysParameter/?repage";
    }
	
	
	

}