/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workcontent.web;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import com.jeeplus.modules.sys.entity.MainDictDetail;
import com.jeeplus.modules.sys.utils.DictUtils;
import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.workcontent.entity.WorkContentBudget;
import com.jeeplus.modules.workcontent.service.WorkContentBudgetService;

/**
 * 结算编审Controller
 * @author ssrh
 * @version 2018-06-06
 */
@Controller
@RequestMapping(value = "${adminPath}/workcontent/workContentBudget")
public class WorkContentBudgetController extends BaseController {

	@Autowired
	private WorkContentBudgetService workContentBudgetService;
	
	@ModelAttribute
	public WorkContentBudget get(@RequestParam(required=false) String id) {
		WorkContentBudget entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = workContentBudgetService.get(id);
		}
		if (entity == null){
			entity = new WorkContentBudget();
		}
		return entity;
	}
	
	/**
	 * 结算编审列表页面
	 */
	@RequiresPermissions("workcontent:workContentBudget:list")
	@RequestMapping(value = {"list", ""})
	public String list(WorkContentBudget workContentBudget,HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<WorkContentBudget> page = workContentBudgetService.findPage(new Page<WorkContentBudget>(request, response), workContentBudget); 
		model.addAttribute("page", page);
		return "modules/workcontent/workContentBudgetList";
	}

	/**
	 * 查看，增加，编辑结算编审表单页面
	 */
	@RequestMapping(value = "form")
	public String form(WorkContentBudget workContentBudget, String view, Model model) {
        List<WorkContentBudget> budgetList = new ArrayList<>();
        if(StringUtils.isNotBlank(workContentBudget.getContentId())){
            budgetList = workContentBudgetService.findList(workContentBudget);
        }
		model.addAttribute("budgetList", budgetList);
        String viewPath = "modules/workcontent/workContentBudgetList";
        if("view".equals(view)){
            viewPath+="View";
        }
		List<MainDictDetail> list=DictUtils.getMainDictList("node_type");
		model.addAttribute("list", list);
        return viewPath;
	}

	/**
	 * 保存结算编审
	 */
	@RequiresPermissions(value={"workcontent:workContentBudget:add","workcontent:workContentBudget:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(WorkContentBudget workContentBudget, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, workContentBudget)){
			return form(workContentBudget, null,model);
		}
		if(!workContentBudget.getIsNewRecord()){//编辑表单保存
			WorkContentBudget t = workContentBudgetService.get(workContentBudget.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(workContentBudget, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			workContentBudgetService.save(t);//保存
		}else{//新增表单保存
			workContentBudgetService.save(workContentBudget);//保存
		}
		addMessage(redirectAttributes, "保存结算编审成功");
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentBudget/?repage";
	}
	
	/**
	 * 删除结算编审
	 */
	@RequiresPermissions("workcontent:workContentBudget:del")
	@RequestMapping(value = "delete")
	public String delete(WorkContentBudget workContentBudget, RedirectAttributes redirectAttributes) {
		workContentBudgetService.delete(workContentBudget);
		addMessage(redirectAttributes, "删除结算编审成功");
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentBudget/?repage";
	}
	
	/**
	 * 批量删除结算编审
	 */
	@RequiresPermissions("workcontent:workContentBudget:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			workContentBudgetService.delete(workContentBudgetService.get(id));
		}
		addMessage(redirectAttributes, "删除结算编审成功");
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentBudget/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("workcontent:workContentBudget:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(WorkContentBudget workContentBudget, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "结算编审"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<WorkContentBudget> page = workContentBudgetService.findPage(new Page<WorkContentBudget>(request, response, -1), workContentBudget);
    		new ExportExcel("结算编审", WorkContentBudget.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出结算编审记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentBudget/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("workcontent:workContentBudget:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<WorkContentBudget> list = ei.getDataList(WorkContentBudget.class);
			for (WorkContentBudget workContentBudget : list){
				try{
					workContentBudgetService.save(workContentBudget);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条结算编审记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条结算编审记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入结算编审失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentBudget/?repage";
    }
	
	/**
	 * 下载导入结算编审数据模板
	 */
	@RequiresPermissions("workcontent:workContentBudget:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "结算编审数据导入模板.xlsx";
    		List<WorkContentBudget> list = Lists.newArrayList(); 
    		new ExportExcel("结算编审数据", WorkContentBudget.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentBudget/?repage";
    }
	
	
	

}