/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workcontent.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.workcontent.entity.WorkContentFundplan;
import com.jeeplus.modules.workcontent.service.WorkContentFundplanService;

/**
 * 资金计划Controller
 * @author ssrh
 * @version 2018-06-06
 */
@Controller
@RequestMapping(value = "${adminPath}/workcontent/workContentFundplan")
public class WorkContentFundplanController extends BaseController {

	@Autowired
	private WorkContentFundplanService workContentFundplanService;
	
	@ModelAttribute
	public WorkContentFundplan get(@RequestParam(required=false) String id) {
		WorkContentFundplan entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = workContentFundplanService.get(id);
		}
		if (entity == null){
			entity = new WorkContentFundplan();
		}
		return entity;
	}
	
	/**
	 * 资金计划列表页面
	 */
	@RequestMapping(value = {"list", ""})
	public String list(WorkContentFundplan workContentFundplan, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<WorkContentFundplan> page = workContentFundplanService.findPage(new Page<WorkContentFundplan>(request, response), workContentFundplan); 
		model.addAttribute("page", page);
		return "modules/workcontent/workContentFundplanList";
	}

	/**
	 * 查看，增加，编辑资金计划表单页面
	 */
	@RequestMapping(value = "form")
	public String form(WorkContentFundplan workContentFundplan,String view, Model model) {
        WorkContentFundplan oldRecord = null;
        if(StringUtils.isNotBlank(workContentFundplan.getContentId())){
            List<WorkContentFundplan> list = workContentFundplanService.findList(workContentFundplan);
            if(list!=null&&!list.isEmpty()){
                oldRecord = list.get(0);
            }
        }
        if(oldRecord!=null){
            model.addAttribute("workContentFundplan", oldRecord);
        }else {
            model.addAttribute("workContentFundplan", workContentFundplan);
        }
        String viewPath = "modules/workcontent/workContentFundplan";
        if("view".equals(view)){
            viewPath+="View";
        }
        return viewPath;
	}

	/**
	 * 保存资金计划
	 */
	@RequestMapping(value = "save")
	public String save(WorkContentFundplan workContentFundplan, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, workContentFundplan)){
			return form(workContentFundplan,null, model);
		}
		if(!workContentFundplan.getIsNewRecord()){//编辑表单保存
			WorkContentFundplan t = workContentFundplanService.get(workContentFundplan.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(workContentFundplan, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			workContentFundplanService.save(t);//保存
		}else{//新增表单保存
			workContentFundplanService.save(workContentFundplan);//保存
		}
		addMessage(redirectAttributes, "保存资金计划成功");
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentFundplan/?repage";
	}
	
	/**
	 * 删除资金计划
	 */
	@RequestMapping(value = "delete")
	public String delete(WorkContentFundplan workContentFundplan, RedirectAttributes redirectAttributes) {
		workContentFundplanService.delete(workContentFundplan);
		addMessage(redirectAttributes, "删除资金计划成功");
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentFundplan/?repage";
	}
	
	/**
	 * 批量删除资金计划
	 */
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			workContentFundplanService.delete(workContentFundplanService.get(id));
		}
		addMessage(redirectAttributes, "删除资金计划成功");
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentFundplan/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(WorkContentFundplan workContentFundplan, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "资金计划"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<WorkContentFundplan> page = workContentFundplanService.findPage(new Page<WorkContentFundplan>(request, response, -1), workContentFundplan);
    		new ExportExcel("资金计划", WorkContentFundplan.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出资金计划记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentFundplan/?repage";
    }

	/**
	 * 导入Excel数据

	 */
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<WorkContentFundplan> list = ei.getDataList(WorkContentFundplan.class);
			for (WorkContentFundplan workContentFundplan : list){
				try{
					workContentFundplanService.save(workContentFundplan);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条资金计划记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条资金计划记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入资金计划失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentFundplan/?repage";
    }
	
	/**
	 * 下载导入资金计划数据模板
	 */
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "资金计划数据导入模板.xlsx";
    		List<WorkContentFundplan> list = Lists.newArrayList(); 
    		new ExportExcel("资金计划数据", WorkContentFundplan.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentFundplan/?repage";
    }
	
	
	

}