/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workcontent.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.workcontent.entity.WorkContentPriceadjust;
import com.jeeplus.modules.workcontent.service.WorkContentPriceadjustService;

/**
 * 动态成本控制Controller
 * @author ssrh
 * @version 2018-06-11
 */
@Controller
@RequestMapping(value = "${adminPath}/workcontent/workContentPriceadjust")
public class WorkContentPriceadjustController extends BaseController {

	@Autowired
	private WorkContentPriceadjustService workContentPriceadjustService;
	
	@ModelAttribute
	public WorkContentPriceadjust get(@RequestParam(required=false) String id) {
		WorkContentPriceadjust entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = workContentPriceadjustService.get(id);
		}
		if (entity == null){
			entity = new WorkContentPriceadjust();
		}
		return entity;
	}
	
	/**
	 * 动态成本控制列表页面
	 */
	@RequestMapping(value = {"list", ""})
	public String list(WorkContentPriceadjust workContentPriceadjust, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<WorkContentPriceadjust> page = workContentPriceadjustService.findPage(new Page<WorkContentPriceadjust>(request, response), workContentPriceadjust); 
		model.addAttribute("page", page);
		return "modules/workcontent/workContentPriceadjustList";
	}

	/**
	 * 查看，增加，编辑动态成本控制表单页面
	 */
	@RequestMapping(value = "form")
	public String form(WorkContentPriceadjust workContentPriceadjust,String view, Model model) {
        WorkContentPriceadjust oldRecord = null;
        if(StringUtils.isNotBlank(workContentPriceadjust.getContentId())){
            List<WorkContentPriceadjust> list = workContentPriceadjustService.findList(workContentPriceadjust);
            if(list!=null&&!list.isEmpty()){
                oldRecord = list.get(0);
            }
        }
        if(oldRecord!=null){
            model.addAttribute("workContentPriceadjust", oldRecord);
        }else {
            model.addAttribute("workContentPriceadjust", workContentPriceadjust);
        }
        String viewPath = "modules/workcontent/workContentPriceadjustForm";
        if("view".equals(view)){
            viewPath+="View";
        }
        return viewPath;
	}

	/**
	 * 保存动态成本控制
	 */
	@RequestMapping(value = "save")
	public String save(WorkContentPriceadjust workContentPriceadjust, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, workContentPriceadjust)){
			return form(workContentPriceadjust, null,model);
		}
		if(!workContentPriceadjust.getIsNewRecord()){//编辑表单保存
			WorkContentPriceadjust t = workContentPriceadjustService.get(workContentPriceadjust.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(workContentPriceadjust, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			workContentPriceadjustService.save(t);//保存
		}else{//新增表单保存
			workContentPriceadjustService.save(workContentPriceadjust);//保存
		}
		addMessage(redirectAttributes, "保存动态成本控制成功");
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentPriceadjust/?repage";
	}
	
	/**
	 * 删除动态成本控制
	 */
	@RequestMapping(value = "delete")
	public String delete(WorkContentPriceadjust workContentPriceadjust, RedirectAttributes redirectAttributes) {
		workContentPriceadjustService.delete(workContentPriceadjust);
		addMessage(redirectAttributes, "删除动态成本控制成功");
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentPriceadjust/?repage";
	}
	
	/**
	 * 批量删除动态成本控制
	 */
	@RequiresPermissions("workcontent:workContentPriceadjust:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			workContentPriceadjustService.delete(workContentPriceadjustService.get(id));
		}
		addMessage(redirectAttributes, "删除动态成本控制成功");
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentPriceadjust/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("workcontent:workContentPriceadjust:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(WorkContentPriceadjust workContentPriceadjust, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "动态成本控制"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<WorkContentPriceadjust> page = workContentPriceadjustService.findPage(new Page<WorkContentPriceadjust>(request, response, -1), workContentPriceadjust);
    		new ExportExcel("动态成本控制", WorkContentPriceadjust.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出动态成本控制记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentPriceadjust/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("workcontent:workContentPriceadjust:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<WorkContentPriceadjust> list = ei.getDataList(WorkContentPriceadjust.class);
			for (WorkContentPriceadjust workContentPriceadjust : list){
				try{
					workContentPriceadjustService.save(workContentPriceadjust);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条动态成本控制记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条动态成本控制记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入动态成本控制失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentPriceadjust/?repage";
    }
	
	/**
	 * 下载导入动态成本控制数据模板
	 */
	@RequiresPermissions("workcontent:workContentPriceadjust:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "动态成本控制数据导入模板.xlsx";
    		List<WorkContentPriceadjust> list = Lists.newArrayList(); 
    		new ExportExcel("动态成本控制数据", WorkContentPriceadjust.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentPriceadjust/?repage";
    }
	
	
	

}