/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workcontent.web;

import java.util.Date;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import com.jeeplus.modules.sys.utils.UserUtils;
import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.workcontent.entity.WorkContentProjectprice;
import com.jeeplus.modules.workcontent.service.WorkContentProjectpriceService;

/**
 * 工程价款管理Controller
 * @author 工程价款管理
 * @version 2018-06-13
 */
@Controller
@RequestMapping(value = "${adminPath}/workcontent/workContentProjectprice")
public class WorkContentProjectpriceController extends BaseController {

	@Autowired
	private WorkContentProjectpriceService workContentProjectpriceService;
	
	@ModelAttribute
	public WorkContentProjectprice get(@RequestParam(required=false) String id) {
		WorkContentProjectprice entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = workContentProjectpriceService.get(id);
		}
		if (entity == null){
			entity = new WorkContentProjectprice();
		}
		return entity;
	}
	
	/**
	 * 工程价款管理列表页面
	 */
	@RequiresPermissions("workcontent:workContentProjectprice:list")
	@RequestMapping(value = {"list", ""})
	public String list(WorkContentProjectprice workContentProjectprice, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<WorkContentProjectprice> page = workContentProjectpriceService.findPage(new Page<WorkContentProjectprice>(request, response), workContentProjectprice); 
		model.addAttribute("page", page);
		return "modules/workcontent/workContentProjectpriceList";
	}

	/**
	 * 查看，增加，编辑工程价款管理表单页面
	 */
	@RequestMapping(value = "form")
	public String form(WorkContentProjectprice workContentProjectprice,String view, Model model) {
        WorkContentProjectprice oldRecord = null;
        if(StringUtils.isNotBlank(workContentProjectprice.getContentId())){
            List<WorkContentProjectprice> list = workContentProjectpriceService.findList(workContentProjectprice);
            if(list!=null&&!list.isEmpty()){
                oldRecord = list.get(0);
            }
        }
        if(oldRecord!=null){
            model.addAttribute("workContentProjectprice", oldRecord);
        }else {
            workContentProjectprice.setCreateBy(UserUtils.getUser());
            workContentProjectprice.setCreateDate(new Date());
            model.addAttribute("workContentProjectprice", workContentProjectprice);
        }
        String viewPath = "modules/workcontent/workContentProjectpriceForm";
        if("view".equals(view)){
            viewPath+="View";
        }
        return viewPath;
	}

	/**
	 * 保存工程价款管理
	 */
	@RequiresPermissions(value={"workcontent:workContentProjectprice:add","workcontent:workContentProjectprice:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(WorkContentProjectprice workContentProjectprice, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, workContentProjectprice)){
			return form(workContentProjectprice, null,model);
		}
		if(!workContentProjectprice.getIsNewRecord()){//编辑表单保存
			WorkContentProjectprice t = workContentProjectpriceService.get(workContentProjectprice.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(workContentProjectprice, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			workContentProjectpriceService.save(t);//保存
		}else{//新增表单保存
			workContentProjectpriceService.save(workContentProjectprice);//保存
		}
		addMessage(redirectAttributes, "保存工程价款管理成功");
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentProjectprice/?repage";
	}
	
	/**
	 * 删除工程价款管理
	 */
	@RequiresPermissions("workcontent:workContentProjectprice:del")
	@RequestMapping(value = "delete")
	public String delete(WorkContentProjectprice workContentProjectprice, RedirectAttributes redirectAttributes) {
		workContentProjectpriceService.delete(workContentProjectprice);
		addMessage(redirectAttributes, "删除工程价款管理成功");
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentProjectprice/?repage";
	}
	
	/**
	 * 批量删除工程价款管理
	 */
	@RequiresPermissions("workcontent:workContentProjectprice:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			workContentProjectpriceService.delete(workContentProjectpriceService.get(id));
		}
		addMessage(redirectAttributes, "删除工程价款管理成功");
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentProjectprice/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("workcontent:workContentProjectprice:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(WorkContentProjectprice workContentProjectprice, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "工程价款管理"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<WorkContentProjectprice> page = workContentProjectpriceService.findPage(new Page<WorkContentProjectprice>(request, response, -1), workContentProjectprice);
    		new ExportExcel("工程价款管理", WorkContentProjectprice.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出工程价款管理记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentProjectprice/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("workcontent:workContentProjectprice:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<WorkContentProjectprice> list = ei.getDataList(WorkContentProjectprice.class);
			for (WorkContentProjectprice workContentProjectprice : list){
				try{
					workContentProjectpriceService.save(workContentProjectprice);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条工程价款管理记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条工程价款管理记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入工程价款管理失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentProjectprice/?repage";
    }
	
	/**
	 * 下载导入工程价款管理数据模板
	 */
	@RequiresPermissions("workcontent:workContentProjectprice:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "工程价款管理数据导入模板.xlsx";
    		List<WorkContentProjectprice> list = Lists.newArrayList(); 
    		new ExportExcel("工程价款管理数据", WorkContentProjectprice.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentProjectprice/?repage";
    }
	
	
	

}