/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workdevicerecord.web;

import com.google.common.collect.Lists;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.modules.workdevicerecord.entity.WorkDeviceRecord;
import com.jeeplus.modules.workdevicerecord.service.WorkDeviceRecordService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;
import java.util.List;

/**
 * 设备询价记录Controller
 * @author 马鹏波
 * @version 2018-06-05
 */
@Controller
@RequestMapping(value = "${adminPath}/workdevicerecord/workDeviceRecord")
public class WorkDeviceRecordController extends BaseController {

	@Autowired
	private WorkDeviceRecordService workDeviceRecordService;
	
	@ModelAttribute
	public WorkDeviceRecord get(@RequestParam(required=false) String id) {
		WorkDeviceRecord entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = workDeviceRecordService.get(id);
		}
		if (entity == null){
			entity = new WorkDeviceRecord();
		}
		return entity;
	}
	
	/**
	 * 设备询价记录列表页面
	 */
//	@RequiresPermissions("workdevicerecord:workDeviceRecord:list")
	@RequestMapping(value = {"list", ""})
	public String list(WorkDeviceRecord workDeviceRecord, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<WorkDeviceRecord> page = workDeviceRecordService.findPage(new Page<WorkDeviceRecord>(request, response), workDeviceRecord); 
		model.addAttribute("page", page);
		return "modules/workdevicerecord/workDeviceRecordList";
	}

	/**
	 * 查看，增加，编辑设备询价记录表单页面
	 */
//	@RequiresPermissions(value={"workdevicerecord:workDeviceRecord:view","workdevicerecord:workDeviceRecord:add","workdevicerecord:workDeviceRecord:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(WorkDeviceRecord workDeviceRecord, Model model) {
		model.addAttribute("workDeviceRecord", workDeviceRecord);
		return "modules/workdevicerecord/workDeviceRecordForm";
	}

	/**
	 * 保存设备询价记录
	 */
//	@RequiresPermissions(value={"workdevicerecord:workDeviceRecord:add","workdevicerecord:workDeviceRecord:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(WorkDeviceRecord workDeviceRecord, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, workDeviceRecord)){
			return form(workDeviceRecord, model);
		}
		if(!workDeviceRecord.getIsNewRecord()){//编辑表单保存
			WorkDeviceRecord t = workDeviceRecordService.get(workDeviceRecord.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(workDeviceRecord, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			workDeviceRecordService.save(t);//保存
		}else{//新增表单保存
			workDeviceRecordService.save(workDeviceRecord);//保存
		}
		addMessage(redirectAttributes, "保存设备询价记录成功");
		return "redirect:"+Global.getAdminPath()+"/workdevicerecord/workDeviceRecord/?repage";
	}
	
	/**
	 * 删除设备询价记录
	 */
//	@RequiresPermissions("workdevicerecord:workDeviceRecord:del")
	@RequestMapping(value = "delete")
	public String delete(WorkDeviceRecord workDeviceRecord, RedirectAttributes redirectAttributes) {
		workDeviceRecordService.delete(workDeviceRecord);
		addMessage(redirectAttributes, "删除设备询价记录成功");
		return "redirect:"+Global.getAdminPath()+"/workdevicerecord/workDeviceRecord/?repage";
	}
	
	/**
	 * 批量删除设备询价记录
	 */
//	@RequiresPermissions("workdevicerecord:workDeviceRecord:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			workDeviceRecordService.delete(workDeviceRecordService.get(id));
		}
		addMessage(redirectAttributes, "删除设备询价记录成功");
		return "redirect:"+Global.getAdminPath()+"/workdevicerecord/workDeviceRecord/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
//	@RequiresPermissions("workdevicerecord:workDeviceRecord:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(WorkDeviceRecord workDeviceRecord, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "设备询价记录"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<WorkDeviceRecord> page = workDeviceRecordService.findPage(new Page<WorkDeviceRecord>(request, response, -1), workDeviceRecord);
    		new ExportExcel("设备询价记录", WorkDeviceRecord.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出设备询价记录记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workdevicerecord/workDeviceRecord/?repage";
    }

	/**
	 * 导入Excel数据

	 */
//	@RequiresPermissions("workdevicerecord:workDeviceRecord:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<WorkDeviceRecord> list = ei.getDataList(WorkDeviceRecord.class);
			for (WorkDeviceRecord workDeviceRecord : list){
				try{
					workDeviceRecordService.save(workDeviceRecord);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条设备询价记录记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条设备询价记录记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入设备询价记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workdevicerecord/workDeviceRecord/?repage";
    }
	
	/**
	 * 下载导入设备询价记录数据模板
	 */
//	@RequiresPermissions("workdevicerecord:workDeviceRecord:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "设备询价记录数据导入模板.xlsx";
    		List<WorkDeviceRecord> list = Lists.newArrayList(); 
    		new ExportExcel("设备询价记录数据", WorkDeviceRecord.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workdevicerecord/workDeviceRecord/?repage";
    }
	
	
	

}