/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workdevicerecord.web;

import com.google.common.base.Strings;
import com.google.common.collect.Lists;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.modules.workdevicerecord.entity.WorkDeviceRecord;
import com.jeeplus.modules.workdevicerecord.entity.WorkDeviceXjRecord;
import com.jeeplus.modules.workdevicerecord.service.WorkDeviceRecordService;
import com.jeeplus.modules.workdevicerecord.service.WorkDeviceXjRecordService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;
import java.util.List;

/**
 * 设备询价记录Controller
 * @author 马鹏波
 * @version 2018-06-05
 */
@Controller
@RequestMapping(value = "${adminPath}/workdevicerecord/workDeviceXjRecord")
public class WorkDeviceXjRecordController extends BaseController {

	@Autowired
	private WorkDeviceXjRecordService workDeviceXjRecordService;
	@Autowired
	private WorkDeviceRecordService workDeviceRecordService;
	
	@ModelAttribute
	public WorkDeviceXjRecord get(@RequestParam(required=false) String id) {
		WorkDeviceXjRecord entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = workDeviceXjRecordService.get(id);
		}
		if (entity == null){
			entity = new WorkDeviceXjRecord();
		}
		return entity;
	}
	
	/**
	 * 设备询价记录列表页面
	 */
//	@RequiresPermissions("workdevicerecord:workDeviceXjRecord:list")
	@RequestMapping(value = {"list", ""})
	public String list(WorkDeviceXjRecord workDeviceXjRecord, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<WorkDeviceXjRecord> page = workDeviceXjRecordService.findPage(new Page<WorkDeviceXjRecord>(request, response), workDeviceXjRecord); 
		model.addAttribute("page", page);
		return "modules/workdevicerecord/workDeviceXjRecordList";
	}

	/**
	 * 查看，增加，编辑设备询价记录表单页面
	 */
//	@RequiresPermissions(value={"workdevicerecord:workDeviceXjRecord:view","workdevicerecord:workDeviceXjRecord:add","workdevicerecord:workDeviceXjRecord:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(WorkDeviceXjRecord workDeviceXjRecord, HttpServletRequest request,Model model) {
		String view = request.getParameter("view");
		if("view".equals(view)){
			view = "modules/workdevicerecord/workDeviceXjRecordFormView";
		}else{
			view = "modules/workdevicerecord/workDeviceXjRecordForm";
		}
		if(!Strings.isNullOrEmpty(workDeviceXjRecord.getProjectId())){
			workDeviceXjRecord = workDeviceXjRecordService.findListByXjId(workDeviceXjRecord);
			if(!workDeviceXjRecord.getIsNewRecord()){
				WorkDeviceRecord workDeviceRecord = new WorkDeviceRecord();
				workDeviceRecord.setDeviceXjId(workDeviceXjRecord.getId());
				List<WorkDeviceRecord> listByXjId = workDeviceRecordService.findListByXjId(workDeviceRecord);
				workDeviceXjRecord.setWorkDeviceRecordList(listByXjId);
			}
		}
		model.addAttribute("workDeviceXjRecord", workDeviceXjRecord);
		return view;
	}

	/**
	 * 保存设备询价记录
	 */
//	@RequiresPermissions(value={"workdevicerecord:workDeviceXjRecord:add","workdevicerecord:workDeviceXjRecord:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(WorkDeviceXjRecord workDeviceXjRecord, HttpServletRequest request,Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, workDeviceXjRecord)){
			return form(workDeviceXjRecord, request,model);
		}
		if(!workDeviceXjRecord.getIsNewRecord()){//编辑表单保存
			WorkDeviceXjRecord t = workDeviceXjRecordService.get(workDeviceXjRecord.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(workDeviceXjRecord, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			workDeviceXjRecordService.save(t);//保存
		}else{//新增表单保存
			workDeviceXjRecordService.save(workDeviceXjRecord);//保存
		}
		addMessage(redirectAttributes, "保存设备询价记录成功");
		return "redirect:"+Global.getAdminPath()+"/workdevicerecord/workDeviceXjRecord/?repage";
	}
	
	/**
	 * 删除设备询价记录
	 */
//	@RequiresPermissions("workdevicerecord:workDeviceXjRecord:del")
	@RequestMapping(value = "delete")
	public String delete(WorkDeviceXjRecord workDeviceXjRecord, RedirectAttributes redirectAttributes) {
		workDeviceXjRecordService.delete(workDeviceXjRecord);
		addMessage(redirectAttributes, "删除设备询价记录成功");
		return "redirect:"+Global.getAdminPath()+"/workdevicerecord/workDeviceXjRecord/?repage";
	}
	
	/**
	 * 批量删除设备询价记录
	 */
//	@RequiresPermissions("workdevicerecord:workDeviceXjRecord:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			workDeviceXjRecordService.delete(workDeviceXjRecordService.get(id));
		}
		addMessage(redirectAttributes, "删除设备询价记录成功");
		return "redirect:"+Global.getAdminPath()+"/workdevicerecord/workDeviceXjRecord/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
//	@RequiresPermissions("workdevicerecord:workDeviceXjRecord:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(WorkDeviceXjRecord workDeviceXjRecord, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "设备询价记录"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<WorkDeviceXjRecord> page = workDeviceXjRecordService.findPage(new Page<WorkDeviceXjRecord>(request, response, -1), workDeviceXjRecord);
    		new ExportExcel("设备询价记录", WorkDeviceXjRecord.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出设备询价记录记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workdevicerecord/workDeviceXjRecord/?repage";
    }

	/**
	 * 导入Excel数据

	 */
//	@RequiresPermissions("workdevicerecord:workDeviceXjRecord:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<WorkDeviceXjRecord> list = ei.getDataList(WorkDeviceXjRecord.class);
			for (WorkDeviceXjRecord workDeviceXjRecord : list){
				try{
					workDeviceXjRecordService.save(workDeviceXjRecord);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条设备询价记录记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条设备询价记录记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入设备询价记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workdevicerecord/workDeviceXjRecord/?repage";
    }
	
	/**
	 * 下载导入设备询价记录数据模板
	 */
//	@RequiresPermissions("workdevicerecord:workDeviceXjRecord:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "设备询价记录数据导入模板.xlsx";
    		List<WorkDeviceXjRecord> list = Lists.newArrayList(); 
    		new ExportExcel("设备询价记录数据", WorkDeviceXjRecord.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workdevicerecord/workDeviceXjRecord/?repage";
    }
	
	
	

}