/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workstaff.service;

import java.io.InputStream;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.jeeplus.common.bos.BOSClientUtil;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.oss.OSSClientUtil;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.IdGen;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.modules.sys.dao.UserDao;
import com.jeeplus.modules.sys.entity.User;
import com.jeeplus.modules.sys.utils.DictUtils;
import com.jeeplus.modules.workstaff.entity.WorkStaffBasicInfo;
import com.jeeplus.modules.workstaff.entity.WorkStaffEducation;
import com.jeeplus.modules.workstaffachiveslog.entity.WorkStaffAchivesLog;
import com.jeeplus.modules.workstaffachiveslog.service.WorkStaffAchivesLogService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.service.CrudService;
import com.jeeplus.modules.workstaff.entity.WorkStaffTitle;
import com.jeeplus.modules.workstaff.dao.WorkStaffTitleDao;
import org.springframework.web.multipart.MultipartFile;

/**
 * 职称Service
 * @author ssrh
 * @version 2018-07-27
 */
@Service
@Transactional(readOnly = true)
public class WorkStaffTitleService extends CrudService<WorkStaffTitleDao, WorkStaffTitle> {
    @Autowired
    private WorkStaffAchivesLogService workStaffAchivesLogService;
    @Autowired
    private UserDao userDao;
	public WorkStaffTitle get(String id) {
		return super.get(id);
	}
	
	public List<WorkStaffTitle> findList(WorkStaffTitle workStaffTitle) {
		return super.findList(workStaffTitle);
	}
	
	public Page<WorkStaffTitle> findPage(Page<WorkStaffTitle> page, WorkStaffTitle workStaffTitle) {
		return super.findPage(page, workStaffTitle);
	}
	
	@Transactional(readOnly = false)
	public void save(WorkStaffTitle workStaffTitle) {
		super.save(workStaffTitle);
	}
	
	@Transactional(readOnly = false)
	public void delete(WorkStaffTitle workStaffTitle) {
		super.delete(workStaffTitle);
	}


    public boolean save(WorkStaffBasicInfo workStaffBasicInfo,boolean first) {
        if(workStaffBasicInfo==null|| StringUtils.isBlank(workStaffBasicInfo.getId())){
            return false;
        }
        List<WorkStaffTitle> list = workStaffBasicInfo.getTitleList();
        if(list==null||list.size()<=0){
            return false;
        }
        boolean flag=false;
        for (WorkStaffTitle entity : list) {
            if(entity!=null&&"1".equals(entity.getDelFlag())&&StringUtils.isBlank(entity.getId())){
                continue;
            }
            if(entity!=null&&"1".equals(entity.getDelFlag())){
                dao.deleteByLogic(entity);
                continue;
            }
            if (entity!=null){
                if(entity.getFile()!=null&&!entity.getFile().isEmpty()&&entity.getFile().getSize()>0){
                    MultipartFile file = entity.getFile();
                    entity.setFileName(file.getOriginalFilename());
                    entity.setFilePath(this.uploadFile(file,entity.getFilePath()));
                }
                entity.setStaffId(workStaffBasicInfo.getId());
                this.save(entity);
                if(first){
                    entity.setId(null);
                    entity.setFilePath("");
                }
                flag = true;
            }
        }
        return flag;
    }

    public String uploadFile(MultipartFile file, String filePath){
        String date = DateUtils.formatDate(new Date(),"yyMMddHHmm");
        try {
            if (file != null && file.getSize() > 0) {
                String path = new StringBuilder("/employeeInfo/").append(date)
                        .append(file.getOriginalFilename()).toString();
                InputStream inputStream = file.getInputStream();
                BOSClientUtil bosClientUtil = new BOSClientUtil();
                String url = bosClientUtil.upload(path, inputStream);
                return url;
            }
        }catch (Exception e){
            logger.error("上传文件失败！");
        }
        return null;
    }

    public void query(WorkStaffBasicInfo workStaffBasicInfo) {
        if(workStaffBasicInfo==null|| StringUtils.isBlank(workStaffBasicInfo.getId())){
            return;
        }
        WorkStaffTitle entity = new WorkStaffTitle();
        entity.setStaffId(workStaffBasicInfo.getId());
        workStaffBasicInfo.setTitleList(this.findList(entity));
    }

    private static Map<String,String> EDU_MAP = Maps.newHashMap();//教育经历

    static{//教育经历
        EDU_MAP.put("name", "职称名称");
        EDU_MAP.put("grade", "职称等级");
        EDU_MAP.put("startDate", "取得日期");
        EDU_MAP.put("pathway", "取得途径");
        EDU_MAP.put("approvalAuthority", "审批单位");
        EDU_MAP.put("startDate", "入学日期");
        EDU_MAP.put("filePath", "文件");
        EDU_MAP.put("fileName", "文件名");
    }

    public boolean saveApply(WorkStaffBasicInfo workStaffBasicInfo,boolean first) {
        if(workStaffBasicInfo==null|| StringUtils.isBlank(workStaffBasicInfo.getId())){
            return false;
        }
        List<WorkStaffTitle> list = workStaffBasicInfo.getTitleList();
        if(list==null||list.size()<=0){
            return false;
        }
        boolean flag = false;
        User u=userDao.get(workStaffBasicInfo.getUserId());
        for (WorkStaffTitle entity : list) {
            entity.setStaffId(workStaffBasicInfo.getId());
            WorkStaffTitle workStaffTitle=dao.get(entity.getId());
            if(workStaffTitle==null){
                entity.setCreateBy(u);
                entity.setCreateDate(new Date());
                entity.setUpdateBy(u);
                entity.setUpdateDate(new Date());
                dao.insert(entity);
            }else{
                this.save(entity);
            }
            if(entity!=null&&"1".equals(entity.getDelFlag())){
                dao.deleteByLogic(entity);
                continue;
            }
            flag =true;
        }
        dao.updateFalt(workStaffBasicInfo.getId());
        return flag;
    }
    public void querys(WorkStaffBasicInfo workStaffBasicInfo) {
        SimpleDateFormat dateFormat1 = new SimpleDateFormat("yyyy-MM-dd");
        if(workStaffBasicInfo==null|| StringUtils.isBlank(workStaffBasicInfo.getId())){
            return;
        }
        WorkStaffTitle entity = new WorkStaffTitle();
        entity.setStaffId(workStaffBasicInfo.getId());
        List<WorkStaffTitle> list=this.findList(entity);
        List<WorkStaffTitle> newList=Lists.newArrayList();
        if(list!=null&&list.size()>0){//修改
            for(WorkStaffTitle workStaffTitle:list){
                Class<? extends WorkStaffTitle> newClass = workStaffTitle.getClass();
                Field[] declaredFields = newClass.getDeclaredFields();
                for (Field field : declaredFields) {
                    String name = field.getName();
                    if("name".equals(name)||"grade".equals(name)||"startDate".equals(name)||"pathway".equals(name)||"approvalAuthority".equals(name)
                            || "startDate".equals(name)||"filePath".equals(name)||"fileName".equals(name)){
                        String setMethod = "set"+name.substring(0,1).toUpperCase()+name.substring(1);
                        String getMethod = "get"+name.substring(0,1).toUpperCase()+name.substring(1);
                        WorkStaffAchivesLog workStaffAchivesLog=new WorkStaffAchivesLog();
                        workStaffAchivesLog.setStaffId(workStaffBasicInfo.getId());
                        workStaffAchivesLog.setModule("职称");
                        workStaffAchivesLog.setSonId(workStaffTitle.getId());
                        workStaffAchivesLog.setFields(name);
                        workStaffAchivesLog.setType("修改");
                        workStaffAchivesLog.setState("1");
                        List<WorkStaffAchivesLog> logs=workStaffAchivesLogService.findList(workStaffAchivesLog);
                        try {
                            Object newInvoke = newClass.getMethod(getMethod).invoke(workStaffTitle);//修改后
                            if(newInvoke instanceof String){
                                Method setReadOnly = workStaffTitle.getClass().getMethod(setMethod, String.class);
                                if(logs!=null&&logs.size()>0){
                                    setReadOnly.invoke(workStaffTitle,logs.get(0).getNewKey());
                                }
                            }
                            if(newInvoke instanceof Date){//日期
                                Method setReadOnly = workStaffTitle.getClass().getMethod(setMethod, Date.class);
                                if(logs!=null&&logs.size()>0){
                                    Date myDate = dateFormat1.parse(logs.get(0).getNewKey());
                                    setReadOnly.invoke(workStaffTitle,myDate);
                                }
                            }
                        } catch (NoSuchMethodException e) {
                            e.printStackTrace();
                        } catch (IllegalAccessException e) {
                            e.printStackTrace();
                        } catch (InvocationTargetException e) {
                            e.printStackTrace();
                        } catch (ParseException e) {
                            e.printStackTrace();
                        }
                    }
                }
                newList.add(workStaffTitle);
            }
        }
        //新增
        WorkStaffAchivesLog workStaffAchivesLog=new WorkStaffAchivesLog();
        workStaffAchivesLog.setStaffId(workStaffBasicInfo.getId());
        workStaffAchivesLog.setModule("职称");
        workStaffAchivesLog.setType("新增");
        workStaffAchivesLog.setState("1");
        List<WorkStaffAchivesLog> sonIds=workStaffAchivesLogService.findSonId(workStaffAchivesLog);
        if(sonIds!=null&&sonIds.size()>0){
            for(WorkStaffAchivesLog w:sonIds){
                workStaffAchivesLog.setSonId(w.getSonId());
                WorkStaffTitle workStaffTitle = new WorkStaffTitle();
                workStaffTitle.setId(w.getSonId());
                workStaffTitle.setDelFlag("0");
                workStaffTitle.setStaffId(workStaffBasicInfo.getId());
                Class<? extends WorkStaffTitle> newClass = workStaffTitle.getClass();
                Field[] declaredFields = newClass.getDeclaredFields();
                for (Field field : declaredFields) {
                    String name = field.getName();
                    if("name".equals(name)||"grade".equals(name)||"startDate".equals(name)||"pathway".equals(name)||"approvalAuthority".equals(name)
                            || "startDate".equals(name)||"filePath".equals(name)||"fileName".equals(name)){
                        String setMethod = "set"+name.substring(0,1).toUpperCase()+name.substring(1);
                        String getMethod = "get"+name.substring(0,1).toUpperCase()+name.substring(1);
                        workStaffAchivesLog.setFields(name);
                        List<WorkStaffAchivesLog> logs=workStaffAchivesLogService.findList(workStaffAchivesLog);
                        try {
                            if("startDate".equals(name)){//日期
                                Method setReadOnly = workStaffTitle.getClass().getMethod(setMethod, Date.class);
                                if(logs!=null&&logs.size()>0){
                                    Date myDate = dateFormat1.parse(logs.get(0).getNewKey());
                                    setReadOnly.invoke(workStaffTitle,myDate);
                                }
                            }else{
                                Method setReadOnly = workStaffTitle.getClass().getMethod(setMethod, String.class);
                                if(logs!=null&&logs.size()>0){
                                    setReadOnly.invoke(workStaffTitle,logs.get(0).getNewKey());
                                }
                            }
                        } catch (NoSuchMethodException e) {
                            e.printStackTrace();
                        } catch (IllegalAccessException e) {
                            e.printStackTrace();
                        } catch (InvocationTargetException e) {
                            e.printStackTrace();
                        } catch (ParseException e) {
                            e.printStackTrace();
                        }
                    }
                }
                newList.add(workStaffTitle);
            }
        }
        //删除
        WorkStaffAchivesLog delLog=new WorkStaffAchivesLog();
        delLog.setStaffId(workStaffBasicInfo.getId());
        delLog.setModule("职称");
        delLog.setType("删除");
        workStaffAchivesLog.setState("1");
        List<WorkStaffAchivesLog> dellogs=workStaffAchivesLogService.findList(delLog);
        if(dellogs!=null&&dellogs.size()>0){
            for(WorkStaffAchivesLog w:dellogs){
                newList.remove(dao.get(w.getSonId()));
            }
        }
        workStaffBasicInfo.setTitleList(newList);
    }

    @Transactional(readOnly = false)
    public void saveEdu(WorkStaffBasicInfo workStaffBasicInfo) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        List<WorkStaffTitle> titleList =workStaffBasicInfo.getTitleList();
        if(titleList!=null&&titleList.size()>0){
            for(WorkStaffTitle newTitle:titleList){
                Class<? extends WorkStaffTitle> newClass = newTitle.getClass();
                Field[] declaredFields = newClass.getDeclaredFields();
                if(newTitle.getId()!=null&&!"1".equals(newTitle.getDelFlag())&&!"".equals(newTitle.getId())){
                    if(newTitle.getFile()!=null&&!newTitle.getFile().isEmpty()&&newTitle.getFile().getSize()>0){
                        MultipartFile file = newTitle.getFile();
                        newTitle.setFilePath(this.uploadFile(file,newTitle.getFilePath()));
                    }
                    WorkStaffTitle oldTitle=this.get(newTitle.getId());
                    if(oldTitle!=null){
                        Class<? extends WorkStaffTitle> oldClass = oldTitle.getClass();
                        for (Field field : declaredFields) {
                            String name = field.getName();
                            if("name".equals(name)||"grade".equals(name)||"startDate".equals(name)||"pathway".equals(name)||"approvalAuthority".equals(name)
                                    || "startDate".equals(name)||"filePath".equals(name)||"fileName".equals(name)){
                                String methodName = "get"+name.substring(0,1).toUpperCase()+name.substring(1);
                                WorkStaffAchivesLog workStaffAchivesLog=new WorkStaffAchivesLog();
                                workStaffAchivesLog.setStaffId(workStaffBasicInfo.getId());
                                workStaffAchivesLog.setModule("职称");
                                workStaffAchivesLog.setSonId(newTitle.getId());
                                workStaffAchivesLog.setFields(name);
                                workStaffAchivesLog.setState("1");
                                try {
                                    Object newInvoke = newClass.getMethod(methodName).invoke(newTitle);//修改后
                                    Object oldInvoke = oldClass.getMethod(methodName).invoke(oldTitle);//修改前
                                    String describes=EDU_MAP.get(name);
                                    if(newInvoke instanceof String){
                                        String newValue=newInvoke.toString();
                                        String oldValue=oldInvoke.toString();
                                        if(!newValue.equals(oldValue)){
                                                workStaffAchivesLogService.saveStaffLog(workStaffAchivesLog,workStaffBasicInfo.getId(),
                                                        "职称",describes,name,
                                                        oldInvoke.toString(),newInvoke.toString(),
                                                        oldInvoke.toString(),newInvoke.toString(),newTitle.getId(),"修改");

                                        }
                                    }
                                    if(newInvoke instanceof Date){//日期
                                        String newDate="";
                                        String oldDate="";
                                        if(newInvoke!=null){
                                            newDate=sdf.format(newInvoke);
                                        }
                                        if (oldInvoke!=null){
                                            oldDate=sdf.format(oldInvoke);
                                        }
                                        if(!newDate.equals(oldDate)){
                                            workStaffAchivesLogService.saveStaffLog(workStaffAchivesLog,workStaffBasicInfo.getId(),
                                                    "职称",describes,name,
                                                    oldDate,newDate,oldDate,newDate,newTitle.getId(),"修改");
                                        }
                                    }
                                } catch (NoSuchMethodException e) {
                                    e.printStackTrace();
                                } catch (IllegalAccessException e) {
                                    e.printStackTrace();
                                } catch (InvocationTargetException e) {
                                    e.printStackTrace();
                                }
                            }
                        }

                    }
                }else if((newTitle.getId()==null||"".equals(newTitle.getId()))&&"0".equals(newTitle.getDelFlag())){//新增
                    if(newTitle.getFile()!=null&&!newTitle.getFile().isEmpty()&&newTitle.getFile().getSize()>0){
                        MultipartFile file = newTitle.getFile();
                        newTitle.setFilePath(this.uploadFile(file,newTitle.getFilePath()));
                    }
                    String newId=IdGen.uuid();
                    for (Field field : declaredFields) {
                        String name = field.getName();
                        if("name".equals(name)||"grade".equals(name)||"startDate".equals(name)||"pathway".equals(name)||"approvalAuthority".equals(name)
                                || "startDate".equals(name)||"filePath".equals(name)||"fileName".equals(name)){
                            String methodName = "get"+name.substring(0,1).toUpperCase()+name.substring(1);
                            WorkStaffAchivesLog workStaffAchivesLog=new WorkStaffAchivesLog();
                            workStaffAchivesLog.setStaffId(workStaffBasicInfo.getId());
                            workStaffAchivesLog.setModule("职称");
                            workStaffAchivesLog.setSonId(newId);
                            workStaffAchivesLog.setFields(name);
                            workStaffAchivesLog.setState("1");
                            try {
                                Object newInvoke = newClass.getMethod(methodName).invoke(newTitle);//修改后
                                String describes=EDU_MAP.get(name);
                                if(newInvoke instanceof String){
                                    String newValue=newInvoke.toString();
                                        workStaffAchivesLogService.saveStaffLog(workStaffAchivesLog,workStaffBasicInfo.getId(),
                                                "职称",describes,name,
                                                "",newInvoke.toString(),
                                                "",newInvoke.toString(),newId,"新增");

                                }
                                if(newInvoke instanceof Date){//日期
                                    String newDate=sdf.format(newInvoke);
                                    workStaffAchivesLogService.saveStaffLog(workStaffAchivesLog,workStaffBasicInfo.getId(),
                                            "职称",describes,name,
                                            "",newDate,"",newDate,newId,"新增");
                                };
                            } catch (NoSuchMethodException e) {
                                e.printStackTrace();
                            } catch (IllegalAccessException e) {
                                e.printStackTrace();
                            } catch (InvocationTargetException e) {
                                e.printStackTrace();
                            }
                        }
                    }
                }else if(newTitle.getId()!=null&&"1".equals(newTitle.getDelFlag())&&!"".equals(newTitle.getId())){//删除
                    WorkStaffAchivesLog workStaffAchivesLog=new WorkStaffAchivesLog();
                    workStaffAchivesLog.setStaffId(workStaffBasicInfo.getId());
                    workStaffAchivesLog.setModule("职称");
                    workStaffAchivesLog.setSonId(newTitle.getId());
                    workStaffAchivesLog.setType("删除");
                    workStaffAchivesLog.setState("1");
                    workStaffAchivesLogService.saveStaffLog(workStaffAchivesLog,workStaffBasicInfo.getId(),
                            "职称","","",
                            "","",
                            "","",newTitle.getId(),"删除");
                }
            }
        }
    }
}